
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild::OptionalXS v0.01.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "c_source" => "c",
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28",
    "perl" => "5.020"
  },
  "dist_abstract" => "A linked open code language and processor",
  "dist_author" => [
    "James Smith <jgsmith\@gmail.com>"
  ],
  "dist_name" => "Dallycot",
  "dist_version" => "0.151590",
  "license" => "perl",
  "module_name" => "Dallycot",
  "need_compiler" => 0,
  "recommends" => {},
  "recursive_test_files" => 1,
  "requires" => {
    "AnyEvent" => 7,
    "Array::Utils" => 0,
    "CHI" => "0.59",
    "Data::UUID" => 1,
    "DateTime" => 0,
    "DateTime::Calendar::Hebrew" => 0,
    "DateTime::Calendar::Hijri" => 0,
    "DateTime::Calendar::Julian" => 0,
    "DateTime::Calendar::Mayan" => 0,
    "DateTime::Calendar::Pataphysical" => 0,
    "DateTime::Format::Flexible" => 0,
    "Digest::MD5" => 0,
    "Encode" => 0,
    "Lingua::Sentence" => "1.05",
    "Lingua::StopWords" => "0.09",
    "Lingua::YALI" => "0.015",
    "List::MoreUtils" => "0.23",
    "List::Util" => "1.41",
    "Markdent" => "0.25",
    "Marpa::R2" => 3,
    "Math::BaseCalc" => 0,
    "Math::BigInt::Random" => "0.04",
    "Math::BigRat" => "0.2606",
    "Module::Pluggable" => "5.1",
    "Mojolicious" => "5.65",
    "Moose" => "0.34",
    "MooseX::Getopt" => "0.24",
    "MooseX::Singleton" => "0.04",
    "MooseX::Types::Moose" => 0,
    "Promises" => "0.01",
    "RDF::Query" => "2.912",
    "RDF::RDFa::Parser" => "1.097",
    "RDF::Trine" => "1.011",
    "Readonly" => 0,
    "Scalar::Util" => 0,
    "String::Escape" => 0,
    "Term::ReadLine" => 0,
    "URI" => 0,
    "URI::WithBase" => "2.19",
    "experimental" => "0.013",
    "perl" => "5.020"
  },
  "script_files" => [
    "bin/dallycot"
  ],
  "test_requires" => {
    "EV" => "4.18",
    "perl" => "5.020"
  }
);


my %fallback_build_requires = (
  "EV" => "4.18",
  "Module::Build" => "0.28",
  "perl" => "5.020"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);

my $build_xs;
$build_xs = 0 if grep { $_ eq '--pp' } @ARGV;
$build_xs = 1 if grep { $_ eq '--xs' } @ARGV;
if (!defined $build_xs) {
  $build_xs = $build->have_c_compiler;
}

unless ($build_xs) {
  $build->build_elements(
    [ grep { $_ ne 'xs' } @{ $build->build_elements } ]
  );
}
$build->create_build_script;
