package Dallycot;
$Dallycot::VERSION = '0.151590';
our $AUTHORITY = 'cpan:JSMITH';
use strict;
use warnings;

# ABSTRACT: A linked open code language and processor

use utf8;

use Dallycot::Parser;
use Dallycot::Processor;
use Dallycot::Value;

1;
